@@ -1,6 +1,7 @@
 #' @title Download samBada
 #' @description Downloads from GitHub the version of samBada that corresponds to your OS. Unzips the folder and adds the path to the binary folder to the environmental path variable. This operation is only valid for the current R session. You must run change_path for every new R session. Alternatively, you can manually edit your "PATH" environmental variable permanently on your OS so that it entails the path to the binaries folder of sambada (this procedure different for every OS).
 #' @author Solange Duruz
+#' @details Careful: for MacOS user, they must install GCC7. See sambadoc page 2.
 #' @param directory character The directory where sambada should be downloaded. If null, downloads in a (new) folder named sambada in the active directory.
 #' @examples
 #' # Downloads SamBada to temporary folder (tempdir)
@@ -51,6 +52,7 @@ downloadSambada = function(directory = NULL){
 }
 
 changePath=function(directory){
+  directory=normalizePath(directory)
   #check if directory already in path
   if(grepl(gsub('\\','/',directory, fixed=TRUE),gsub('\\','/',Sys.getenv('PATH'), fixed=TRUE))==FALSE){
     #Add directory to path
@@ -58,16 +60,16 @@ changePath=function(directory){
     if(env=='Windows'){
       Sys.setenv(
         PATH = paste(
+          directory,
           Sys.getenv("PATH"), 
-          directory, 
           sep = ";"
         )
       )
     } else {
       Sys.setenv(
         PATH = paste(
+          directory,
           Sys.getenv("PATH"), 
-          directory, 
           sep = ":"
         )
       )      
